/* input.c */

#include <stdio.h>
#include <stdlib.h>
#include "knnfp.h"

extern char domainName[];
extern char infoFile[];
extern int No_Features, No_Classes;
extern int linear[];
extern int className[], classIndex[];
extern INSTANCE trainData[], testData[];
extern float weight[];

FILE*
check_open_file(char *fileName)
{
  FILE *fp;

  fp=fopen(fileName,"r");
  if (fp==NULL) {
    fprintf(stderr, "%s file is not found...", fileName);
    exit(1);
  }
  return fp;
} /* check_open_file */

separator(char ch)
{
  if (ch == ' ' || ch == ',' || ch == '\t')
    return(1);
  else
    return(0);
}

char*
read_next_value(char *into, char *from)
{
  int j;

  /* skip separators */
  while (separator(*from) && *from != '\n') from++;
  j=0; 
  while (!separator(*from) && *from != '\n') {
    into[j++]=*from; 
    from++; 
  }
  into[j]='\0';			/* into contains a value as a string */
  return from;
} /* read_next_value */

/* read domain dependent parameters from domain.info file */
void 
read_info()
{
  FILE *fp;
  char infoFileName[MAX_NAME], Line[MAX_LINE], str[MAX_LINE], *line;
  FEATURE f; /* to read feature weights */

  sprintf(infoFileName,"%s.info", domainName);
  fp = check_open_file(infoFileName);

  for (f=0; f < MAX_FEATURES; f++) /* initialize feature weights */
     weight[f] = 1.0;

  while (fgets(Line,MAX_LINE,fp) != NULL) {
    if (sscanf(Line,"%s",str) <1) continue; /* ignore empty line */
    if (Line[0]==';') continue;	/* ignore comment line */

    line = read_next_value (str, Line);

    if (strcasecmp(str,"Features")==0) {
      /* read the number and type of features */
      No_Features = 0;
      while ((line=read_next_value(str,line)) && (*str != '\0')) {
	if (No_Features == MAX_FEATURES) {
	  fprintf(stderr, "While reading the %s file number of features exceeded MAX_FEATURES: %d set in cofi.h !", infoFileName, MAX_FEATURES);
	  exit(1);
	}
	if (*str == 'l')
	  linear[No_Features++] = TRUE;
	else
	  linear[No_Features++] = FALSE;
      }
    }
    if ( strcasecmp(str,"Weights") == 0 ) {
      /* read the weights of features */
      if (No_Features == 0) {
	fprintf(stderr, "Please enter the Weights line after the Features line in the ", infoFileName);
	exit(1);
      }
      else 
	for (f=0; f<No_Features; f++) {
	  line=read_next_value(str,line);
	  weight[f] = atof(str);
	}
    }
    else if ( strcasecmp(str,"Classes") == 0 ) {
      /* read number and names of classes */
      No_Classes = 0;
      while ((line=read_next_value(str,line)) && (*str != '\0')) {
	if (No_Classes > MAX_CLASSES) {
	  fprintf(stderr, "While reading the %s file number of classes exceeded MAX_CLASSES: %d set in cofi.h !", infoFileName, MAX_CLASSES);
	  exit(1);
	}
	className[++No_Classes] = atoi(str);
	classIndex[className[No_Classes]] = No_Classes;
	/* 0 < classIndex, 0 is used for UNDETERMINED class */
      }
    }
  }
  fclose(fp);
} /* read_info */

void 
read_an_instance(char *fname, INSTANCE *i, char *line)
{
  char value_in_string[MAX_LINE];
  int v;
  FEATURE f = 0;
  CLASS c = 0;

  /* read feature values */
  for (v=0; v < No_Features; v++) {
    line = read_next_value(value_in_string, line);
    if (value_in_string[0] == '?')
      i->status[f++]=UNKNOWN;
    else { 
      sscanf(value_in_string, "%f", &(i->value[f]));
      i->status[f++]=KNOWN;
    }
  }

  /* read class value */
  line = read_next_value(value_in_string, line);
  if (value_in_string[0] == '?') {
    fprintf(stderr, "Unknown (?) class value in", fname);
    exit(1);
  }
  sscanf(value_in_string, "%d", &c);
  if (classIndex[c] == 0) {
    fprintf(stderr,  "Strange class name or empty line in", fname);
    exit(1);
  }
  i->class = classIndex[c];
} /* read_an_instance */

int 
read_train_data()
{
  FILE *fp;
  int n;
  char trainFileName[MAX_NAME], line[MAX_LINE];

  sprintf(trainFileName,"%s.train", domainName);
  fp = check_open_file(trainFileName);
  
  for (n = 0; fgets(line, MAX_LINE, fp) != 0; n++)
    if (n > MAX_TRAIN) {
      fprintf(stderr, "More train data in %s than MAX_TRAIN=%d",
	      trainFileName, MAX_TRAIN);
      exit(1);
    }
    else read_an_instance(trainFileName, &trainData[n], line);

  fclose(fp);
  return(n);
} /* read_train_data */

int 
read_test_data()
{
  FILE *fp;
  int n;
  char testFileName[MAX_NAME], line[MAX_LINE];

  sprintf(testFileName,"%s.test", domainName);
  fp = check_open_file(testFileName);

  for (n=0; fgets(line, MAX_LINE, fp) != 0; n++)
    if (n == MAX_TEST) {
      fprintf(stderr, "More test data in %s than MAX_TEST=%d",
	      testFileName, MAX_TEST);
      exit(1);
    }
    else read_an_instance(testFileName, &testData[n], line);
  
  fclose(fp);
  return(n);
} /* read_test_data */
